@@ -1,7 +1,7 @@
 # Functions to manipulate Python binary distribution archives.
 #
-# Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: November 28, 2014
+# Author: Peter Odding <peter.odding@paylogic.com>
+# Last Change: October 30, 2015
 # URL: https://github.com/paylogic/pip-accel
 
 """
@@ -13,6 +13,7 @@ binary distribution archives.
 """
 
 # Standard library modules.
+import fnmatch
 import logging
 import os
 import os.path
@@ -38,6 +39,7 @@ from pip_accel.utils import compact, makedirs
 # Initialize a logger for this module.
 logger = logging.getLogger(__name__)
 
+
 class BinaryDistributionManager(object):
 
     """Generates and transforms Python binary distributions."""
@@ -70,28 +72,39 @@ class BinaryDistributionManager(object):
                   :py:class:`tarfile.TarInfo` object and a file-like object.
         """
         cache_file = self.cache.get(requirement)
+        if cache_file and requirement.last_modified > os.path.getmtime(cache_file):
+            logger.info("Invalidating old %s binary (source is newer) ..", requirement)
+            cache_file = None
         if not cache_file:
-            logger.debug("%s (%s) hasn't been cached yet, doing so now.", requirement.name, requirement.version)
+            logger.debug("%s hasn't been cached yet, doing so now.", requirement)
             # Build the binary distribution.
             try:
                 raw_file = self.build_binary_dist(requirement)
             except BuildFailed:
-                logger.warning("Build of %s (%s) failed, checking for missing dependencies ..", requirement.name, requirement.version)
+                logger.warning("Build of %s failed, checking for missing dependencies ..", requirement)
                 if self.system_package_manager.install_dependencies(requirement):
                     raw_file = self.build_binary_dist(requirement)
                 else:
                     raise
             # Transform the binary distribution archive into a form that we can re-use.
-            transformed_file = os.path.join(tempfile.gettempdir(), os.path.basename(raw_file))
-            archive = tarfile.open(transformed_file, 'w:gz')
-            for member, from_handle in self.transform_binary_dist(raw_file):
-                archive.addfile(member, from_handle)
-            archive.close()
-            # Push the binary distribution archive to all available backends.
-            with open(transformed_file, 'rb') as handle:
-                self.cache.put(requirement, handle)
-            # Cleanup the temporary file.
-            os.remove(transformed_file)
+            fd, transformed_file = tempfile.mkstemp(prefix='pip-accel-bdist-', suffix='.tar.gz')
+            try:
+                archive = tarfile.open(transformed_file, 'w:gz')
+                try:
+                    for member, from_handle in self.transform_binary_dist(raw_file):
+                        archive.addfile(member, from_handle)
+                finally:
+                    archive.close()
+                # Push the binary distribution archive to all available backends.
+                with open(transformed_file, 'rb') as handle:
+                    self.cache.put(requirement, handle)
+            finally:
+                # Close file descriptor before removing the temporary file.
+                # Without closing Windows is complaining that the file cannot
+                # be removed because it is used by another process.
+                os.close(fd)
+                # Cleanup the temporary file.
+                os.remove(transformed_file)
             # Get the absolute pathname of the file in the local cache.
             cache_file = self.cache.get(requirement)
         archive = tarfile.open(cache_file, 'r:gz')
@@ -138,9 +151,8 @@ class BinaryDistributionManager(object):
         try:
             return self.build_binary_dist_helper(requirement, ['bdist_dumb', '--format=tar'])
         except (BuildFailed, NoBuildOutput):
-            logger.warning("Build of %s (%s) failed, falling back to alternative method ..",
-                           requirement.name, requirement.version)
-            return self.build_binary_dist_helper(requirement, ['bdist'])
+            logger.warning("Build of %s failed, falling back to alternative method ..", requirement)
+            return self.build_binary_dist_helper(requirement, ['bdist', '--formats=gztar'])
 
     def build_binary_dist_helper(self, requirement, setup_command):
         """
@@ -164,22 +176,35 @@ class BinaryDistributionManager(object):
             msg = "Directory %s (%s %s) doesn't contain a source distribution!"
             raise InvalidSourceDistribution(msg % (requirement.source_directory, requirement.name, requirement.version))
         # Let the user know what's going on.
-        build_text = "Building %s (%s) binary distribution" % (requirement.name, requirement.version)
+        build_text = "Building %s binary distribution" % requirement
         logger.info("%s ..", build_text)
         # Cleanup previously generated distributions.
         dist_directory = os.path.join(requirement.source_directory, 'dist')
         if os.path.isdir(dist_directory):
             logger.debug("Cleaning up previously generated distributions in %s ..", dist_directory)
             shutil.rmtree(dist_directory)
+        # Let the user know (approximately) which command is being executed
+        # (I don't think it's necessary to show them the nasty details :-).
+        logger.debug("Executing external command: %s",
+                     ' '.join(map(pipes.quote, [self.config.python_executable, 'setup.py'] + setup_command)))
         # Compose the command line needed to build the binary distribution.
-        command_line = ' '.join(pipes.quote(t) for t in [self.config.python_executable, 'setup.py'] + setup_command)
-        logger.debug("Executing external command: %s", command_line)
+        # This nasty command line forces the use of setuptools (instead of
+        # distutils) just like pip does. This will cause the `*.egg-info'
+        # metadata to be written to a directory instead of a file, which
+        # (amongst other things) enables tracking of installed files.
+        command_line = [
+            self.config.python_executable, '-c',
+            ';'.join([
+                'import setuptools',
+                '__file__=%r' % setup_script,
+                r"exec(compile(open(__file__).read().replace('\r\n', '\n'), __file__, 'exec'))",
+            ])
+        ] + setup_command
         # Redirect all output of the build to a temporary file.
         fd, temporary_file = tempfile.mkstemp()
-        command_line = '%s > "%s" 2>&1' % (command_line, temporary_file)
         try:
             # Start the build.
-            build = subprocess.Popen(['sh', '-c', command_line], cwd=requirement.source_directory)
+            build = subprocess.Popen(command_line, cwd=requirement.source_directory, stdout=fd, stderr=fd)
             # Wait for the build to finish and provide feedback to the user in the mean time.
             spinner = Spinner(label=build_text, timer=build_timer)
             while build.poll() is None:
@@ -202,8 +227,13 @@ class BinaryDistributionManager(object):
                 # Check if we can find the binary distribution archive.
                 filenames = os.listdir(dist_directory)
                 if len(filenames) != 1:
-                    raise NoBuildOutput("Build of {name} ({version}) produced more than one distribution archive! (matches: {filenames})",
-                                        name=requirement.name, version=requirement.version, filenames=concatenate(sorted(filenames)))
+                    variables = dict(name=requirement.name,
+                                     version=requirement.version,
+                                     filenames=concatenate(sorted(filenames)))
+                    raise NoBuildOutput("""
+                        Build of {name} ({version}) produced more than one
+                        distribution archive! (matches: {filenames})
+                    """, **variables)
             except Exception as e:
                 # Decorate the exception with the output of the failed build.
                 with open(temporary_file) as handle:
@@ -217,13 +247,16 @@ class BinaryDistributionManager(object):
                     Build output:
 
                     {output}
-                """, message=e.args[0],
-                     output=build_output.strip())
+                """, message=e.args[0], output=build_output.strip())
                 e.args = (enhanced_message,)
                 raise
-            logger.info("Finished building %s (%s) in %s.", requirement.name, requirement.version, build_timer)
+            logger.info("Finished building %s in %s.", requirement.name, build_timer)
             return os.path.join(dist_directory, filenames[0])
         finally:
+            # Close file descriptor before removing the temporary file.
+            # Without closing Windows is complaining that the file cannot
+            # be removed because it is used by another process.
+            os.close(fd)
             os.unlink(temporary_file)
 
     def transform_binary_dist(self, archive_path):
@@ -256,18 +289,31 @@ class BinaryDistributionManager(object):
                 # If the owner doesn't have execute permissions we'll give everyone
                 # read permissions (only the owner gets write permissions).
                 member.mode = 0o644
-            # In my testing the `dumb' tar files created with the `python setup.py
-            # bdist' command contain pathnames that are relative to `/' which is
-            # kind of awkward: I would like to use os.path.relpath() on them but
-            # that won't give the correct result without some preprocessing...
+            # In my testing the `dumb' tar files created with the `python
+            # setup.py bdist' and `python setup.py bdist_dumb' commands contain
+            # pathnames that are relative to `/' in one way or another:
+            #
+            #  - In almost all cases the pathnames look like this:
+            #
+            #      ./home/peter/.virtualenvs/pip-accel/lib/python2.7/site-packages/pip_accel/__init__.py
+            #
+            #  - After working on pip-accel for several years I encountered
+            #    a pathname like this (Python 2.6 on Mac OS X 10.10.5):
+            #
+            #      Users/peter/.virtualenvs/pip-accel/lib/python2.6/site-packages/pip_accel/__init__.py
+            #
+            # Both of the above pathnames are relative to `/' but in different
+            # ways :-). The following normpath(join('/', ...))) pathname
+            # manipulation logic is intended to handle both cases.
             original_pathname = member.name
-            absolute_pathname = re.sub(r'^\./', '/', original_pathname)
+            absolute_pathname = os.path.normpath(os.path.join('/', original_pathname))
             if member.isdev():
                 logger.warn("Ignoring device file: %s.", absolute_pathname)
             elif not member.isdir():
                 modified_pathname = os.path.relpath(absolute_pathname, self.config.install_prefix)
                 if os.path.isabs(modified_pathname):
-                    logger.warn("Failed to transform pathname in binary distribution to relative path! (original: %r, modified: %r)",
+                    logger.warn("Failed to transform pathname in binary distribution"
+                                " to relative path! (original: %r, modified: %r)",
                                 original_pathname, modified_pathname)
                 else:
                     # Rewrite /usr/local to /usr (same goes for all prefixes of course).
@@ -280,12 +326,13 @@ class BinaryDistributionManager(object):
                     logger.debug("Transformed %r -> %r.", original_pathname, modified_pathname)
                     # Get the file data from the input archive.
                     handle = archive.extractfile(original_pathname)
-                    # Yield the pathname, file mode and a handle to the data.
+                    # Yield the modified metadata and a handle to the data.
                     member.name = modified_pathname
                     yield member, handle
         archive.close()
 
-    def install_binary_dist(self, members, virtualenv_compatible=True, prefix=None, python=None):
+    def install_binary_dist(self, members, virtualenv_compatible=True, prefix=None,
+                            python=None, track_installed_files=False):
         """
         Install a binary distribution created by :py:class:`build_binary_dist()`
         into the given prefix (a directory like ``/usr``, ``/usr/local`` or a
@@ -306,6 +353,11 @@ class BinaryDistributionManager(object):
                                       resulting filenames compatible with
                                       virtual environments (defaults to
                                       ``True``).
+        :param track_installed_files: If this is ``True`` (not the default for
+                                      this method because of backwards
+                                      compatibility) pip-accel will create
+                                      ``installed-files.txt`` as required by
+                                      pip to properly uninstall packages.
         """
         # TODO This is quite slow for modules like Django. Speed it up! Two choices:
         #  1. Run the external tar program to unpack the archive. This will
@@ -316,6 +368,7 @@ class BinaryDistributionManager(object):
         module_search_path = set(map(os.path.normpath, sys.path))
         prefix = os.path.normpath(prefix or self.config.install_prefix)
         python = os.path.normpath(python or self.config.python_executable)
+        installed_files = []
         for member, from_handle in members:
             pathname = member.name
             if virtualenv_compatible:
@@ -338,6 +391,9 @@ class BinaryDistributionManager(object):
                     if dist_packages in module_search_path and site_packages not in module_search_path:
                         pathname = pathname.replace('/site-packages/', '/dist-packages/')
             pathname = os.path.join(prefix, pathname)
+            if track_installed_files:
+                # Track the installed file's absolute pathname.
+                installed_files.append(pathname)
             directory = os.path.dirname(pathname)
             if not os.path.isdir(directory):
                 logger.debug("Creating directory: %s ..", directory)
@@ -349,6 +405,8 @@ class BinaryDistributionManager(object):
                     contents = self.fix_hashbang(contents, python)
                 to_handle.write(contents)
             os.chmod(pathname, member.mode)
+        if track_installed_files:
+            self.update_installed_files(installed_files)
 
     def fix_hashbang(self, contents, python):
         """
@@ -376,3 +434,28 @@ class BinaryDistributionManager(object):
                 logger.debug("Rewriting hashbang %r to %r!", hashbang, lines[0])
                 contents = b'\n'.join(lines)
         return contents
+
+    def update_installed_files(self, installed_files):
+        """
+        Track the files installed by a package so pip knows how to remove the package.
+
+        This method is used by :py:func:`install_binary_dist()` (which collects
+        the list of installed files for :py:func:`update_installed_files()`).
+
+        :param installed_files: A list of absolute pathnames (strings) with the
+                                files that were just installed.
+        """
+        # Find the *.egg-info directory where installed-files.txt should be created.
+        pkg_info_files = [fn for fn in installed_files if fnmatch.fnmatch(fn, '*.egg-info/PKG-INFO')]
+        # I'm not (yet) sure how reliable the above logic is, so for now
+        # I'll err on the side of caution and only act when the results
+        # seem to be reliable.
+        if len(pkg_info_files) != 1:
+            logger.warning("Not tracking installed files (couldn't reliably determine *.egg-info directory)")
+        else:
+            egg_info_directory = os.path.dirname(pkg_info_files[0])
+            installed_files_path = os.path.join(egg_info_directory, 'installed-files.txt')
+            logger.debug("Tracking installed files in %s ..", installed_files_path)
+            with open(installed_files_path, 'w') as handle:
+                for pathname in installed_files:
+                    handle.write('%s\n' % os.path.relpath(pathname, egg_info_directory))
